;nyquist plug-in
;version 1
;type process
;categories "http://lv2plug.in/ns/lv2core/#"
;name "L..."
;action "Processing"
;info "Ls voice simulation by Steve Daulton (www.easyspacepro.com).\nReleased under GPL v2.\n"

;; by Steve Daulton (www.easyspacepro.com), 2011.
;; Released under terms of the GNU General Public License version 2:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html

;control ps "Overall Pitch Shift" real "ratio" 0.0 -0.5 0.5
;control dly "Delay Multiplier" real "" 2.0 0.1 4.0
;control rev "Reverb Level" real "[dB]" -12 -60 -6
;control time "Reverb Time" real "[seconds]" 2 0.5 5
;control amp "Overall Amplification" real "" 1.2 0.0 2.0



; ---------- 'reverb.lsp' by Roger Dannenberg -------------
(defun reverb (x time) 
  (multichan-expand #'reverb-mono x time))

(defun reverb-mono (ga irevfactor)
  (let (sr ilowpass idel ihz icsc acomball allp1 allp2 allp3 alow allp4 allp5
        arevout)
    (setf sr (snd-srate ga))

    (setf ilowpass 9000.000)       ; frequency of lowpass filter

    (setf idel (list ; list of frequency/delay values
                    (/ 1237.000 sr) (/  1381.000 sr) (/ 1607.000 sr)
                    (/ 1777.000 sr) (/ 1949.000 sr) (/  2063.000 sr)
                    (/ 307.000 sr) (/ 97.000 sr) (/ 71.000 sr)
                    (/ 53.000 sr) (/ 47.000 sr) (/ 37.000 sr)
                    (/ 31.000 sr)))
    ; Nyquist's comb filter uses Hz rather than delay as parameter,
    ; so take reciprocals to get Hz:
    (setf ihz (mapcar #'/ idel))

    (setf icsc (list ; list of delay times
                    (* irevfactor 0.822) (* irevfactor 0.802)
                    (* irevfactor 0.773) (* irevfactor 0.753)
                    (* irevfactor 0.753) (* irevfactor 0.753)
                    (* irevfactor 0.7)))

    (setf acomball (sum
                        (comb ga (nth 0 icsc) (nth 0 ihz))
                        (comb ga (nth 1 icsc) (nth 1 ihz))
                        (comb ga (nth 2 icsc) (nth 2 ihz))
                        (comb ga (nth 3 icsc) (nth 3 ihz))
                        (comb ga (nth 4 icsc) (nth 4 ihz))
                        (comb ga (nth 5 icsc) (nth 5 ihz))))

    (setf allp1 (alpass acomball (nth 6 icsc) (nth 6 ihz)))
    (setf allp2 (alpass allp1 (nth 6 icsc) (nth 7 ihz)))
    (setf allp3 (alpass allp2 (nth 6 icsc) (nth 8 ihz)))
    (setf alow  (lp allp3 ilowpass))
    (setf allp4 (alpass alow (nth 6 icsc) (nth 9 ihz)))
    (setf allp5 (alpass allp4 (nth 6 icsc) (nth 11 ihz)))

    allp5
    ; acomball
    ))
; ------------- End of 'reverb.lsp' by Roger Dannenberg ------------

(setf wet-rev (db-to-linear (- rev 10.0)))
(setf dry-rev (- 1.0 (* 1.414 wet-rev)))

(defun effect-mixer (sound)
  (sum (scale dry-rev sound)
       (scale wet-rev (reverb sound time))))

(defun xreverb (s-in)
(if (arrayp s-in)
    (vector 
      (effect-mixer (aref s-in 0))
      (effect-mixer (aref s-in 1)))
  (effect-mixer s-in)))
  

(hp
  (xreverb
    (let ((v1 (+ ps 0.7))
          (v2 (+ ps 0.8))
          (v3 (+ ps 1.0))
          (v4 (+ ps 1.1)))

      (mult amp 0.2 (sim 
        (mult 3.0 s)
        (pitshift s v1 1.0)
        (pitshift s v2 1.0)
        (pitshift s v3 1.0)
        (pitshift s v4 1.0)

        (at-abs (* dly 0.01) (cue (scale 0.75 (pitshift s v1 1.0))))
        (at-abs (* dly 0.01) (cue (scale 0.75 (pitshift s v2 1.0))))
        (at-abs (* dly 0.01) (cue (scale 0.75 (pitshift s v3 1.0))))
        (at-abs (* dly 0.01) (cue (scale 0.75 (pitshift s v4 1.0))))

        (at-abs (* dly 0.02) (cue (scale 0.5 (pitshift s v1 1.0))))
        (at-abs (* dly 0.02) (cue (scale 0.5 (pitshift s v2 1.0))))
        (at-abs (* dly 0.02) (cue (scale 0.5 (pitshift s v3 1.0))))
        (at-abs (* dly 0.02) (cue (scale 0.5 (pitshift s v4 1.0))))

        (at-abs (* dly 0.03) (cue (scale 0.3 (pitshift s v1 1.0))))
        (at-abs (* dly 0.03) (cue (scale 0.3 (pitshift s v2 1.0))))
        (at-abs (* dly 0.03) (cue (scale 0.3 (pitshift s v3 1.0))))
        (at-abs (* dly 0.03) (cue (scale 0.3 (pitshift s v4 1.0))))
        ))))
40)
